/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import static java.util.Objects.*;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;

@Component
class AfmAntragstellerMapper implements AfmEngineBasedMapper {

	static final String POSTFACH_ID = "nameid";

	static final String ANTRAGSTELLER = "antragsteller";
	static final String ANTRAGSTELLER_UPPERCASE = "Antragsteller";
	static final String ANREDE = "b_anrede";
	static final String VORNAME = "pers_vorname";
	static final String NACHNAME = "pers_nachname";
	static final String GEBURTSNAME = "pers_geburtsname";
	static final String GEBURTSDATUM = "pers_geburtsdatum";
	static final String GEBURTSORT = "pers_geburtsort";
	static final String EMAIL = "kont_email";
	static final String TELEFON = "kont_telefonnummer";
	static final String STRASSE = "sh_strasse";
	static final String HAUSNUMMER = "sh_hausnummer";
	static final String PLZ = "sh_plz";
	static final String ORT = "ort";

	@Autowired
	private AfmAntragstellerHeaderMapper antragstellerHeaderMapper;

	@Override
	public FormData parseFormData(FormData formData) {
		if (antragstellerHeaderMapper.isResponsible(formData)) {
			return antragstellerHeaderMapper.parseAntragstellerData(formData);
		}
		var formDataMap = formData.getFormData();
		var builder = Antragsteller.builder().postfachId(getPostfachId(formDataMap));

		var antragsteller = getAntragstellerMap(formDataMap)
				.map(antragstellerMap -> addAntragstellerData(builder, antragstellerMap))
				.orElse(builder)
				.build();

		var cleanedMap = removeMappedData(formDataMap);
		return formData.toBuilder().antragsteller(antragsteller).formData(cleanedMap).build();
	}

	private String getPostfachId(Map<String, Object> formDataMap) {
		return (String) formDataMap.get(POSTFACH_ID);
	}

	@SuppressWarnings("unchecked")
	private Optional<Map<String, Object>> getAntragstellerMap(Map<String, Object> formDataMap) {
		return Optional.ofNullable(formDataMap.get(ANTRAGSTELLER))
				.or(() -> Optional.ofNullable(formDataMap.get(ANTRAGSTELLER_UPPERCASE)))
				.map(Map.class::cast).map(LinkedHashMap::new);
	}

	private Antragsteller.AntragstellerBuilder addAntragstellerData(Antragsteller.AntragstellerBuilder builder,
			Map<String, Object> antragstellerMap) {
		return builder //
				.anrede((String) antragstellerMap.get(ANREDE)) //
				.vorname((String) antragstellerMap.get(VORNAME)) //
				.nachname((String) antragstellerMap.get(NACHNAME)) //
				.geburtsname((String) antragstellerMap.get(GEBURTSNAME)) //
				.geburtsdatum((String) antragstellerMap.get(GEBURTSDATUM)) //
				.geburtsort((String) antragstellerMap.get(GEBURTSORT)) //
				.email((String) antragstellerMap.get(EMAIL)) //
				.telefon((String) antragstellerMap.get(TELEFON)) //
				.strasse((String) antragstellerMap.get(STRASSE)) //
				.hausnummer((String) antragstellerMap.get(HAUSNUMMER)) //
				.plz((String) antragstellerMap.get(PLZ)) //
				.ort((String) antragstellerMap.get(ORT)) //
				.data(getNotMappedData(antragstellerMap));
	}

	private Map<String, Object> getNotMappedData(Map<String, Object> antragstellerMap) {
		var map = new LinkedHashMap<>(antragstellerMap);
		getFields().forEach(map::remove);
		return map;
	}

	private List<String> getFields() {
		return List.of(ANREDE, VORNAME, NACHNAME, GEBURTSNAME, GEBURTSDATUM, GEBURTSORT, EMAIL, TELEFON, STRASSE, HAUSNUMMER, PLZ, ORT);
	}

	private Map<String, Object> removeMappedData(Map<String, Object> formDataMap) {
		var editableMap = new LinkedHashMap<>(formDataMap);
		if (nonNull(editableMap.get(ANTRAGSTELLER))) {
			editableMap.remove(ANTRAGSTELLER);
		} else {
			editableMap.remove(ANTRAGSTELLER_UPPERCASE);
		}
		return Collections.unmodifiableMap(editableMap);
	}
}